package cn.timer.api.controller.disk;

import cn.timer.api.bean.disk.DiskCatalogue;
import cn.timer.api.config.annotation.CurrentUser;
import cn.timer.api.config.annotation.UserBean;
import cn.timer.api.config.exception.CustomException;
import cn.timer.api.controller.disk.sevice.DiskCatalogueService;
import cn.timer.api.dto.disk.DiskCatalogueDto;
import cn.timer.api.dto.disk.DiskCatalogueParam;
import cn.timer.api.utils.Result;
import cn.timer.api.utils.ResultUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.transaction.Transactional;
import java.util.*;


/**
 * 云盘-资源目录表
 *
 * @author wuqingjun
 * @email 284718418@qq.com
 * @date 2021-12-27 10:05:49
 */
@Api(tags = "云盘")
@Transactional(rollbackOn = Exception.class)
@RestController
@RequestMapping(value = "/disk/catalogue", produces = {"application/json"})
public class DiskCatalogueController {

    @Autowired
    private DiskCatalogueService diskCatalogueService;

    /**
     * 云盘-新建文件夹
     *
     * @param userBean
     * @return
     */
    @PostMapping(value = "/add_catalogue")
    @ApiOperation(value = "1.新建文件夹或共享空间", httpMethod = "POST", notes = "新建文件夹或共享空间：type=0共享空间，1文件夹")
    @ApiOperationSupport(order = 1)
    public Result<Object> addcatalogue(@CurrentUser UserBean userBean, @Validated @RequestBody DiskCatalogueDto diskCatalogueDto) {
        try{
            DiskCatalogue diskCatalogue = new DiskCatalogue();
            DiskCatalogue catalogue = DiskCatalogue.builder().id(diskCatalogueDto.getParentId()).build().selectById();
            Integer empNum = userBean.getEmpNum();
            Integer orgCode = userBean.getOrgCode();
            BeanUtils.copyProperties(diskCatalogueDto, diskCatalogue);
            String parentIds = "";
            if (!StringUtils.isEmpty(catalogue)) {
                parentIds = catalogue.getParentIds();
                diskCatalogue.setParentIds(catalogue.getParentId() > 0 ? parentIds + "," + catalogue.getId() : catalogue.getId().toString());
            }else {
                diskCatalogue.setParentIds(parentIds);
            }
            diskCatalogue.setCreateUserId(empNum);
            diskCatalogue.setOrgId(orgCode);
            Boolean count = diskCatalogue.insertOrUpdate();
            if (count) {
                return ResultUtil.data(diskCatalogue, "新建文件夹成功!");
            } else {
                return ResultUtil.error("新建文件夹失败!");
            }
        }catch (Exception e){
            e.printStackTrace();
            throw new CustomException("新建文件夹或共享空间异常");
        }
    }

    /**
     * 云盘-我的文件文件夹或共享空间列表
     *
     * @param userBean
     * @param type
     * @return
     */
    @GetMapping(value = "/myfile_list")
    @ApiOperation(value = "2.我的文件文件夹或共享空间列表", httpMethod = "GET", notes = "文件夹或共享空间：type=0共享空间，1文件夹")
    @ApiOperationSupport(order = 2)
    public Result<Object> myfilelist(@CurrentUser UserBean userBean,
                                     @ApiParam("文件文件夹或共享空间ID") @RequestParam(required = true, defaultValue = "0") Integer type) {
        Integer empNum = userBean.getEmpNum();
        Integer orgCode = userBean.getOrgCode();
        List<DiskCatalogue> list = diskCatalogueService.getDiskCatalogueList(orgCode, empNum, type);
        return ResultUtil.data(list);
    }

    /**
     * 云盘-我的文件列表
     *
     * @param userBean
     * @param diskCatalogueParam
     * @return
     */
    @GetMapping(value = "/myfile")
    @ApiOperation(value = "我的文件夹列表", httpMethod = "GET", notes = "文件夹或共享空间：type=0共享空间，1文件夹")
    @ApiOperationSupport(order = 2)
    public Result<Object> myfile(@CurrentUser UserBean userBean, DiskCatalogueParam diskCatalogueParam) {
        try {
            Integer empNum = userBean.getEmpNum();
            Integer orgCode = userBean.getOrgCode();
            // 获取文件夹
            List<DiskCatalogue> diskFileDataList = DiskCatalogue.builder().build().selectList(new QueryWrapper<DiskCatalogue>()
                    .lambda().eq(DiskCatalogue::getOrgId, orgCode)
                    .eq(DiskCatalogue::getCreateUserId, empNum)
                    .eq(DiskCatalogue::getParentId, diskCatalogueParam.getParentId())
                    .eq(DiskCatalogue::getType, diskCatalogueParam.getType())
                    .eq(DiskCatalogue::getDeleteFlag, 0)
            );
            // 获取面包屑
            List<DiskCatalogue> breadcrumbDataList = null;

            DiskCatalogue catalogue = DiskCatalogue.builder().id(diskCatalogueParam.getParentId()).build().selectById();

            if(!StringUtils.isEmpty(catalogue)){
                List<Integer> ids = new ArrayList<>();
                if(!StringUtils.isEmpty(catalogue.getParentIds())){
                    String[] list = catalogue.getParentIds().split(",");
                    for (String str :list) {
                        ids.add(Integer.parseInt(str));
                    }
                }
                ids.add(catalogue.getId());
                breadcrumbDataList = DiskCatalogue.builder().build().selectList(new QueryWrapper<DiskCatalogue>()
                        .lambda().in(DiskCatalogue::getId,ids)
                        .eq(DiskCatalogue::getOrgId, orgCode)
                        .eq(DiskCatalogue::getCreateUserId, empNum)
                        .eq(DiskCatalogue::getDeleteFlag, 0)
                );
            }
            Map<String,Object> map = new HashMap<>(2);
            map.put("diskFileData",diskFileDataList);
            map.put("breadcrumbData",breadcrumbDataList);
            return ResultUtil.data(map);
        }catch (Exception e){
            e.printStackTrace();
            throw new CustomException("获取我的文件夹列表异常");
        }
    }
}
